#![allow(unused)]

use esp_idf_sys as _; // If using the `binstart` feature of `esp-idf-sys`, always keep this module imported
use esp_idf_hal::{
    self,
    prelude::Peripherals, 
    units::Hertz,
    i2c::{I2cConfig, I2cDriver} 
};

use ssd1306::{prelude::*, I2CDisplayInterface, Ssd1306};

mod appstate;

mod error;
use error::Error;

mod gps;
use gps::gpsdata;

mod display;
use display::DisplayError;



fn main() -> Result<(), Error> {
    // It is necessary to call this function once. Otherwise some patches to the runtime
    // implemented by esp-idf-sys might not link properly. See https://github.com/esp-rs/esp-idf-template/issues/71
    esp_idf_sys::link_patches();
    // Bind the log crate to the ESP Logging facilities
    //esp_idf_svc::log::EspLogger::initialize_default();

    // Get Peripherals and sysloop ready
    let peripherals = Peripherals::take().unwrap();
    let pins = peripherals.pins;

    // Define program's pins + addresses
    let scl = pins.gpio22;
    let sda = pins.gpio21;
    let gps_rx  = pins.gpio16;
    let _lcd_address: u8 = 0x3C;
    
    // Setup I2C driver
    let config = I2cConfig::new().baudrate(Hertz(921600));
    let i2c = I2cDriver::new(
        peripherals.i2c0, 
        sda, 
        scl, 
        &config
    ).map_err(|err| Error::EspError(err))?;

    let interface = I2CDisplayInterface::new(i2c);
    let mut display = Ssd1306::new(interface, DisplaySize128x64, DisplayRotation::Rotate0)
        .into_buffered_graphics_mode();
    display.init().map_err(|err| Error::DisplayError(DisplayError::SetupError(err)))?;

    let mut app_state = appstate::AppState::default();
    let mut latest_data = gpsdata::GpsData::default();

    Err(error::Error::DisplayError(DisplayError::DrawingError))

    // loop {

    //     // Clear display ready for next write
    //     let style = MonoTextStyle::new(&FONT_10X20, BinaryColor::On);
    //     display.clear(BinaryColor::Off)
    //     .map_err(|_| Error::DisplayError(DisplayError::DrawingError))?;

    //     // Draw text to Display
    //     Text::with_alignment(
    //         &latest_data.get_speed().as_str(),
    //         Point::new(64, 20),
    //         style,
    //         Alignment::Center,
    //     )
    //     .draw(&mut display)
    //     .map_err(|_| Error::DisplayError(DisplayError::DrawingError))?;

    //     //Flush data to the display
    //     display.flush()
    //     .map_err(|_| Error::DisplayError(DisplayError::FlushError))?;
    // }
}